@@ -9,7 +9,7 @@ __all__ = ['DocumentPointer', 'Pointer', 'PointerToken']
 
 import logging
 from abc import abstractmethod, ABCMeta
-from six import add_metaclass, string_types
+from collections.abc import Mapping, Sequence, MutableSequence
 from .exceptions import ExtractError, RefError, LastElement, OutOfBounds, OutOfRange, WrongType, UnstagedError, ParseError  # noqa
 
 logger = logging.getLogger(__name__)
@@ -66,7 +66,7 @@ class DocumentPointer(object):
         return iter([self.document, self.pointer])
 
     def __eq__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             return other == self.__str__()
         return super(Pointer, self).__eq__(other)
 
@@ -136,7 +136,7 @@ class Pointer(object):
         return iter(self.tokens)
 
     def __eq__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             return other == self.__str__()
         return super(Pointer, self).__eq__(other)
 
@@ -155,8 +155,7 @@ class Pointer(object):
         return '<{}({!r})>'.format(self.__class__.__name__, self.__str__())
 
 
-@add_metaclass(ABCMeta)
-class PointerToken(str):
+class PointerToken(str, metaclass=ABCMeta):
     """
     A single token
     """
@@ -216,17 +215,17 @@ class ChildToken(PointerToken):
         :param bypass_ref: disable JSON Reference errors
         """
         try:
-            if isinstance(obj, dict):
+            if isinstance(obj, Mapping):
                 if not bypass_ref and '$ref' in obj:
                     raise RefError(obj, 'presence of a $ref member')
                 obj = self.extract_mapping(obj)
-            elif isinstance(obj, (list, tuple)):
+            elif isinstance(obj, Sequence) and not isinstance(obj, str):
                 obj = self.extract_sequence(obj)
             else:
                 raise WrongType(obj, '{!r} does not apply '
                                      'for {!r}'.format(str(self), obj))
 
-            if isinstance(obj, dict):
+            if isinstance(obj, Mapping):
                 if not bypass_ref and '$ref' in obj:
                     raise RefError(obj, 'presence of a $ref member')
             return obj
