@@ -4,16 +4,15 @@
 
 """
 
-
-__all__ = ['Provider', 'FilesystemProvider', 'PkgProvider', 'SpecProvider']
-
 import json
 import logging
 import os
 import pkg_resources
 from .bases import Provider
 from .exceptions import NotFound
-from .util import loop
+from pathlib import Path
+
+__all__ = ['Provider', 'FilesystemProvider', 'PkgProvider', 'SpecProvider']
 
 logger = logging.getLogger(__name__)
 
@@ -48,7 +47,6 @@ class PkgProvider(Provider):
         self.providers = providers
         self.loaded = True
 
-
     def __getitem__(self, uri):
         if not self.loaded:
             self.load()
@@ -63,10 +61,15 @@ class PkgProvider(Provider):
         raise NotFound('no providers could return {!r}'.format(uri))
 
     def __iter__(self):
-        raise NotImplementedError
+        if not self.loaded:
+            self.load()
+        for name in self.providers.keys():
+            yield name
 
     def __len__(self):
-        raise NotImplementedError
+        if not self.loaded:
+            self.load()
+        return len(self.providers)
 
 
 class FilesystemProvider(Provider):
@@ -84,25 +87,33 @@ class FilesystemProvider(Provider):
 
     """
 
-    def __init__(self, directory, prefix=None):
+    def __init__(self, directory, prefix=None, aliases=None):
         self.directory = directory
-        if not self.directory.endswith('/'):
-            self.directory += '/'
         self.prefix = prefix or ''
         self.loaded = False
+        self.aliases = aliases or {}
+
+    def _spec_name(self, schema, filename):
+        # Let's assume the schema knows its name more accurately than
+        # its path can provide.
+        if schema.get('id'):
+            return schema['id']
+        else:
+            return filename.as_posix()[len(self.directory):-5].lstrip('/')
 
     @property
     def data(self):
         if not self.loaded:
             data = {}
 
-            l = len(self.directory)
-            for filename in loop(self.directory, '*.json'):
-                spec = filename[l:-5]
-                with open(filename, 'r') as file:
+            for filename in Path(self.directory).glob('**/*.json'):
+                with filename.open() as file:
                     schema = json.load(file)
-                data[spec] = schema
 
+                # Let's assume the schema knows its name more accurately than
+                # its path can provide.
+                spec = self._spec_name(schema, filename)
+                data[spec] = schema
             # set the fallbacks
             for spec in sorted(data.keys(), reverse=True):
                 if spec.startswith('draft-'):
@@ -115,11 +126,13 @@ class FilesystemProvider(Provider):
         return self._data
 
     def __getitem__(self, uri):
+        spec = uri
         if uri.startswith(self.prefix):
             spec = uri[len(self.prefix):]
             if spec.endswith('#'):
                 spec = spec[:-1]
 
+        spec = self.aliases.get(spec, spec)
         try:
             return self.data[spec]
         except (KeyError, UnboundLocalError):
@@ -143,4 +156,36 @@ class SpecProvider(FilesystemProvider):
         base = os.path.realpath(os.path.dirname(misc))
         src = os.path.join(base, 'schemas/')
         prefix = 'http://json-schema.org/'
-        super(SpecProvider, self).__init__(src, prefix)
+        super(SpecProvider, self).__init__(src, prefix, aliases={
+            'hyper-schema': 'draft-04/hyper-schema',
+            'schema': 'draft-04/schema'
+        })
+
+    def _spec_name(self, schema, filename):
+        return filename.as_posix()[len(self.directory):-5].lstrip('/')
+
+
+class ProxyProvider(Provider):
+    def __init__(self, provider):
+        self.provider = provider
+        self.local = {}
+
+    def __getitem__(self, uri):
+        try:
+            return self.local[uri]
+        except KeyError:
+            return self.provider[uri]
+
+    def __setitem__(self, uri, schema):
+        self.local[uri] = schema
+
+    def __iter__(self):
+        keys = set(self.local.keys())
+        keys.update(self.provider.keys())
+        for key in sorted(keys):
+            yield key
+
+    def __len__(self):
+        keys = set(self.local.keys())
+        keys.update(self.provider.keys())
+        return len(keys)
