@@ -6,20 +6,17 @@
 
 from __future__ import absolute_import
 
-__all__ = ['ValidationError', 'Validator', 'ReferenceValidator']
-
 import logging
 from abc import abstractmethod, ABCMeta
-from six import add_metaclass
 from jsonspec.pointer import DocumentPointer
 from .exceptions import ValidationError
 
+__all__ = ['ValidationError', 'Validator', 'ReferenceValidator']
 
 logger = logging.getLogger(__name__)
 
 
-@add_metaclass(ABCMeta)
-class Validator(object):
+class Validator(object, metaclass=ABCMeta):
     """
     The mother of Validators.
     """
