@@ -7,15 +7,11 @@
 
 from __future__ import absolute_import
 
-__all__ = ['compile', 'Draft03Validator']
-
 import logging
-import os.path
 import re
 from copy import deepcopy
 from decimal import Decimal
-from six import integer_types, string_types
-from six.moves.urllib.parse import urljoin
+from urllib.parse import urljoin
 from .bases import ReferenceValidator, Validator
 from .exceptions import CompilationError
 from .factorize import register
@@ -24,8 +20,10 @@ from jsonspec.validators.util import uncamel
 from jsonspec.validators.pointer_util import pointer_join
 from jsonspec import driver as json
 
+__all__ = ['compile', 'Draft03Validator']
+
 sequence_types = (list, set, tuple)
-number_types = (integer_types, float, Decimal)
+number_types = (int, float, Decimal)
 logger = logging.getLogger(__name__)
 
 
@@ -54,7 +52,7 @@ def compile(schema, pointer, context, scope=None):
     attrs = {}
 
     if 'additionalItems' in schm:
-        subpointer = os.path.join(pointer, 'additionalItems')
+        subpointer = pointer_join(pointer, 'additionalItems')
         attrs['additional_items'] = schm.pop('additionalItems')
         if isinstance(attrs['additional_items'], dict):
             compiled = compile(attrs['additional_items'],
@@ -68,7 +66,7 @@ def compile(schema, pointer, context, scope=None):
     if 'additionalProperties' in schm:
         attrs['additional_properties'] = schm.pop('additionalProperties')
         if isinstance(attrs['additional_properties'], dict):
-            subpointer = os.path.join(pointer, 'additionalProperties')
+            subpointer = pointer_join(pointer, 'additionalProperties')
             value = attrs['additional_properties']
             attrs['additional_properties'] = compile(value,
                                                      subpointer,
@@ -83,14 +81,14 @@ def compile(schema, pointer, context, scope=None):
             raise CompilationError('dependencies must be an object', schema)
         for key, value in attrs['dependencies'].items():
             if isinstance(value, dict):
-                subpointer = os.path.join(pointer, 'dependencies', key)
+                subpointer = pointer_join(pointer, 'dependencies', key)
                 attrs['dependencies'][key] = compile(value,
                                                      subpointer,
                                                      context,
                                                      scope)
             elif isinstance(value, sequence_types):
                 continue
-            elif not isinstance(value, string_types):
+            elif not isinstance(value, str):
                 raise CompilationError('dependencies must be an array, object or string', schema)  # noqa
 
     if 'disallow' in schm:
@@ -98,14 +96,14 @@ def compile(schema, pointer, context, scope=None):
         if isinstance(attrs['disallow'], sequence_types):
             for index, value in enumerate(attrs['disallow']):
                 if isinstance(value, dict):
-                    subpointer = os.path.join(pointer, 'disallow', str(index))
+                    subpointer = pointer_join(pointer, 'disallow', index)
                     attrs['disallow'][index] = compile(value,
                                                        subpointer,
                                                        context,
                                                        scope)
-                elif not isinstance(value, string_types):
+                elif not isinstance(value, str):
                     raise CompilationError('disallow must be an object or string', schema)  # noqa
-        elif not isinstance(attrs['disallow'], string_types):
+        elif not isinstance(attrs['disallow'], str):
             raise CompilationError('disallow must be an array or string', schema)  # noqa
 
     if 'divisibleBy' in schm:
@@ -130,7 +128,7 @@ def compile(schema, pointer, context, scope=None):
 
     if 'extends' in schm:
         attrs['extends'] = schm.pop('extends')
-        subpointer = os.path.join(pointer, 'extends')
+        subpointer = pointer_join(pointer, 'extends')
         if isinstance(attrs['extends'], dict):
             attrs['extends'] = compile(attrs['extends'],
                                        subpointer,
@@ -147,11 +145,11 @@ def compile(schema, pointer, context, scope=None):
 
     if 'format' in schm:
         attrs['format'] = schm.pop('format')
-        if not isinstance(attrs['format'], string_types):
+        if not isinstance(attrs['format'], str):
             raise CompilationError('format must be a string', schema)
 
     if 'items' in schm:
-        subpointer = os.path.join(pointer, 'items')
+        subpointer = pointer_join(pointer, 'items')
         attrs['items'] = schm.pop('items')
         if isinstance(attrs['items'], (list, tuple)):
             # each value must be a json schema
@@ -170,12 +168,12 @@ def compile(schema, pointer, context, scope=None):
 
     if 'maxItems' in schm:
         attrs['max_items'] = schm.pop('maxItems')
-        if not isinstance(attrs['max_items'], integer_types):
+        if not isinstance(attrs['max_items'], int):
             raise CompilationError('maxItems must be an integer', schema)
 
     if 'maxLength' in schm:
         attrs['max_length'] = schm.pop('maxLength')
-        if not isinstance(attrs['max_length'], integer_types):
+        if not isinstance(attrs['max_length'], int):
             raise CompilationError('maxLength must be integer', schema)
 
     if 'minimum' in schm:
@@ -185,17 +183,17 @@ def compile(schema, pointer, context, scope=None):
 
     if 'minItems' in schm:
         attrs['min_items'] = schm.pop('minItems')
-        if not isinstance(attrs['min_items'], integer_types):
+        if not isinstance(attrs['min_items'], int):
             raise CompilationError('minItems must be an integer', schema)
 
     if 'minLength' in schm:
         attrs['min_length'] = schm.pop('minLength')
-        if not isinstance(attrs['min_length'], integer_types):
+        if not isinstance(attrs['min_length'], int):
             raise CompilationError('minLength must be integer', schema)
 
     if 'pattern' in schm:
         attrs['pattern'] = schm.pop('pattern')
-        if not isinstance(attrs['pattern'], string_types):
+        if not isinstance(attrs['pattern'], str):
             raise CompilationError('pattern must be a string', schema)
 
     if 'patternProperties' in schm:
@@ -203,7 +201,7 @@ def compile(schema, pointer, context, scope=None):
         if not isinstance(attrs['pattern_properties'], dict):
             raise CompilationError('patternProperties must be an object', schema)  # noqa
         for name, value in attrs['pattern_properties'].items():
-            subpointer = os.path.join(pointer, 'patternProperties', name)
+            subpointer = pointer_join(pointer, 'patternProperties', name)
             attrs['pattern_properties'][name] = compile(value,
                                                         subpointer,
                                                         context,
@@ -214,7 +212,7 @@ def compile(schema, pointer, context, scope=None):
         if not isinstance(attrs['properties'], dict):
             raise CompilationError('properties must be an object', schema)
         for name, value in attrs['properties'].items():
-            subpointer = os.path.join(pointer, 'properties', name)
+            subpointer = pointer_join(pointer, 'properties', name)
             attrs['properties'][name] = compile(value,
                                                 subpointer,
                                                 context,
@@ -230,14 +228,14 @@ def compile(schema, pointer, context, scope=None):
         if isinstance(attrs['type'], sequence_types):
             for index, value in enumerate(attrs['type']):
                 if isinstance(value, dict):
-                    subpointer = os.path.join(pointer, 'type', str(index))
+                    subpointer = pointer_join(pointer, 'type', index)
                     attrs['type'][index] = compile(value,
                                                    subpointer,
                                                    context,
                                                    scope)
-                elif not isinstance(value, string_types):
+                elif not isinstance(value, str):
                     raise CompilationError('type must be an object or string', schema)  # noqa
-        elif not isinstance(attrs['type'], string_types):
+        elif not isinstance(attrs['type'], str):
             raise CompilationError('type must be an array or string', schema)  # noqa
 
     if 'uniqueItems' in schm:
@@ -285,7 +283,7 @@ class Draft03Validator(Validator):
         return isinstance(obj, bool)
 
     def is_integer(self, obj):
-        return isinstance(obj, integer_types) and not isinstance(obj, bool)
+        return isinstance(obj, int) and not isinstance(obj, bool)
 
     def is_null(self, obj):
         return obj is None
@@ -297,7 +295,7 @@ class Draft03Validator(Validator):
         return isinstance(obj, dict)
 
     def is_string(self, obj):
-        return isinstance(obj, string_types)
+        return isinstance(obj, str)
 
     def validate(self, obj, pointer=None):
         """
@@ -409,7 +407,7 @@ class Draft03Validator(Validator):
 
     def validate_enum(self, obj, pointer=None):
         if 'enum' in self.attrs:
-            if not obj in self.attrs['enum']:
+            if obj not in self.attrs['enum']:
                 self.fail('Forbidden value', obj, pointer)
         return obj
 
