@@ -7,15 +7,11 @@
 
 from __future__ import absolute_import
 
-__all__ = ['compile', 'Draft04Validator']
-
 import logging
-import os.path
 import re
 from copy import deepcopy
 from decimal import Decimal
-from six import integer_types, string_types
-from six.moves.urllib.parse import urljoin
+from urllib.parse import urljoin
 from .bases import ReferenceValidator, Validator
 from .exceptions import CompilationError
 from .factorize import register
@@ -24,8 +20,10 @@ from jsonspec.validators.util import uncamel
 from jsonspec.validators.pointer_util import pointer_join
 from jsonspec import driver as json
 
+__all__ = ['compile', 'Draft04Validator']
+
 sequence_types = (list, set, tuple)
-number_types = (integer_types, float, Decimal)
+number_types = (int, float, Decimal)
 logger = logging.getLogger(__name__)
 
 
@@ -54,7 +52,7 @@ def compile(schema, pointer, context, scope=None):
     attrs = {}
 
     if 'additionalItems' in schm:
-        subpointer = os.path.join(pointer, 'additionalItems')
+        subpointer = pointer_join(pointer, 'additionalItems')
         attrs['additional_items'] = schm.pop('additionalItems')
         if isinstance(attrs['additional_items'], dict):
             compiled = compile(attrs['additional_items'],
@@ -66,7 +64,7 @@ def compile(schema, pointer, context, scope=None):
             raise CompilationError('wrong type for {}'.format('additional_items'), schema)  # noqa
 
     if 'additionalProperties' in schm:
-        subpointer = os.path.join(pointer, 'additionalProperties')
+        subpointer = pointer_join(pointer, 'additionalProperties')
         attrs['additional_properties'] = schm.pop('additionalProperties')
         if isinstance(attrs['additional_properties'], dict):
             compiled = compile(attrs['additional_properties'],
@@ -78,7 +76,7 @@ def compile(schema, pointer, context, scope=None):
             raise CompilationError('wrong type for {}'.format('additional_properties'), schema)  # noqa
 
     if 'allOf' in schm:
-        subpointer = os.path.join(pointer, 'allOf')
+        subpointer = pointer_join(pointer, 'allOf')
         attrs['all_of'] = schm.pop('allOf')
         if isinstance(attrs['all_of'], (list, tuple)):
             attrs['all_of'] = [compile(element, subpointer, context, scope) for element in attrs['all_of']]  # noqa
@@ -87,7 +85,7 @@ def compile(schema, pointer, context, scope=None):
             raise CompilationError('wrong type for {}'.format('allOf'), schema)  # noqa
 
     if 'anyOf' in schm:
-        subpointer = os.path.join(pointer, 'anyOf')
+        subpointer = pointer_join(pointer, 'anyOf')
         attrs['any_of'] = schm.pop('anyOf')
         if isinstance(attrs['any_of'], (list, tuple)):
             attrs['any_of'] = [compile(element, subpointer, context, scope) for element in attrs['any_of']]  # noqa
@@ -104,7 +102,7 @@ def compile(schema, pointer, context, scope=None):
             raise CompilationError('dependencies must be an object', schema)
         for key, value in attrs['dependencies'].items():
             if isinstance(value, dict):
-                subpointer = os.path.join(pointer, 'dependencies', key)
+                subpointer = pointer_join(pointer, 'dependencies', key)
                 attrs['dependencies'][key] = compile(value,
                                                      subpointer,
                                                      context,
@@ -120,20 +118,20 @@ def compile(schema, pointer, context, scope=None):
     if 'exclusiveMaximum' in schm:
         attrs['exclusive_maximum'] = schm.pop('exclusiveMaximum')
         if not isinstance(attrs['exclusive_maximum'], bool):
-            raise CompilationError('exclusiveMaximum must be a boolean', schema)
+            raise CompilationError('exclusiveMaximum must be a boolean', schema)  # noqa
 
     if 'exclusiveMinimum' in schm:
         attrs['exclusive_minimum'] = schm.pop('exclusiveMinimum')
         if not isinstance(attrs['exclusive_minimum'], bool):
-            raise CompilationError('exclusiveMinimum must be a boolean', schema)
+            raise CompilationError('exclusiveMinimum must be a boolean', schema)  # noqa
 
     if 'format' in schm:
         attrs['format'] = schm.pop('format')
-        if not isinstance(attrs['format'], string_types):
+        if not isinstance(attrs['format'], str):
             raise CompilationError('format must be a string', schema)
 
     if 'items' in schm:
-        subpointer = os.path.join(pointer, 'items')
+        subpointer = pointer_join(pointer, 'items')
         attrs['items'] = schm.pop('items')
         if isinstance(attrs['items'], (list, tuple)):
             # each value must be a json schema
@@ -152,17 +150,17 @@ def compile(schema, pointer, context, scope=None):
 
     if 'maxItems' in schm:
         attrs['max_items'] = schm.pop('maxItems')
-        if not isinstance(attrs['max_items'], integer_types):
+        if not isinstance(attrs['max_items'], int):
             raise CompilationError('maxItems must be integer', schema)
 
     if 'maxLength' in schm:
         attrs['max_length'] = schm.pop('maxLength')
-        if not isinstance(attrs['max_length'], integer_types):
+        if not isinstance(attrs['max_length'], int):
             raise CompilationError('maxLength must be integer', schema)
 
     if 'maxProperties' in schm:
         attrs['max_properties'] = schm.pop('maxProperties')
-        if not isinstance(attrs['max_properties'], integer_types):
+        if not isinstance(attrs['max_properties'], int):
             raise CompilationError('maxProperties must be integer', schema)
 
     if 'minimum' in schm:
@@ -172,17 +170,17 @@ def compile(schema, pointer, context, scope=None):
 
     if 'minItems' in schm:
         attrs['min_items'] = schm.pop('minItems')
-        if not isinstance(attrs['min_items'], integer_types):
+        if not isinstance(attrs['min_items'], int):
             raise CompilationError('minItems must be integer', schema)
 
     if 'minLength' in schm:
         attrs['min_length'] = schm.pop('minLength')
-        if not isinstance(attrs['min_length'], integer_types):
+        if not isinstance(attrs['min_length'], int):
             raise CompilationError('minLength must be integer', schema)
 
     if 'minProperties' in schm:
         attrs['min_properties'] = schm.pop('minProperties')
-        if not isinstance(attrs['min_properties'], integer_types):
+        if not isinstance(attrs['min_properties'], int):
             raise CompilationError('minProperties must be integer', schema)
 
     if 'multipleOf' in schm:
@@ -194,11 +192,11 @@ def compile(schema, pointer, context, scope=None):
         attrs['not'] = schm.pop('not')
         if not isinstance(attrs['not'], dict):
             raise CompilationError('not must be an object', schema)
-        subpointer = os.path.join(pointer, 'not')
+        subpointer = pointer_join(pointer, 'not')
         attrs['not'] = compile(attrs['not'], subpointer, context, scope)
 
     if 'oneOf' in schm:
-        subpointer = os.path.join(pointer, 'oneOf')
+        subpointer = pointer_join(pointer, 'oneOf')
         attrs['one_of'] = schm.pop('oneOf')
         if isinstance(attrs['one_of'], (list, tuple)):
             # each value must be a json schema
@@ -209,7 +207,7 @@ def compile(schema, pointer, context, scope=None):
 
     if 'pattern' in schm:
         attrs['pattern'] = schm.pop('pattern')
-        if not isinstance(attrs['pattern'], string_types):
+        if not isinstance(attrs['pattern'], str):
             raise CompilationError('pattern must be a string', schema)
 
     if 'properties' in schm:
@@ -224,9 +222,9 @@ def compile(schema, pointer, context, scope=None):
     if 'patternProperties' in schm:
         attrs['pattern_properties'] = schm.pop('patternProperties')
         if not isinstance(attrs['pattern_properties'], dict):
-            raise CompilationError('patternProperties must be an object', schema)
+            raise CompilationError('patternProperties must be an object', schema)  # noqa
         for subname, subschema in attrs['pattern_properties'].items():
-            subpointer = os.path.join(pointer, 'patternProperties', subname)
+            subpointer = pointer_join(pointer, 'patternProperties', subname)
             compiled = compile(subschema, subpointer, context, scope)
             attrs['pattern_properties'][subname] = compiled
 
@@ -239,7 +237,7 @@ def compile(schema, pointer, context, scope=None):
 
     if 'type' in schm:
         attrs['type'] = schm.pop('type')
-        if isinstance(attrs['type'], string_types):
+        if isinstance(attrs['type'], str):
             attrs['type'] = [attrs['type']]
         elif not isinstance(attrs['type'], sequence_types):
             raise CompilationError('type must be string or sequence', schema)
@@ -326,7 +324,6 @@ class Draft04Validator(Validator):
             obj = validator.validate_format(obj, pointer)
 
         if validator.errors:
-            print(validator.errors)
             raise ValidationError('multiple errors',
                                   obj,
                                   errors=validator.errors)
@@ -340,7 +337,7 @@ class Draft04Validator(Validator):
         return isinstance(obj, bool)
 
     def is_integer(self, obj):
-        return isinstance(obj, integer_types) and not isinstance(obj, bool)
+        return isinstance(obj, int) and not isinstance(obj, bool)
 
     def is_number(self, obj):
         return isinstance(obj, number_types) and not isinstance(obj, bool)
@@ -349,7 +346,7 @@ class Draft04Validator(Validator):
         return isinstance(obj, dict)
 
     def is_string(self, obj):
-        return isinstance(obj, string_types)
+        return isinstance(obj, str)
 
     def has_default(self):
         return 'default' in self.attrs
@@ -381,8 +378,8 @@ class Draft04Validator(Validator):
         for key, dependencies in self.attrs.get('dependencies', {}).items():
             if key in obj:
                 if isinstance(dependencies, sequence_types):
-                    for dep in set(dependencies) - set(obj.keys()):
-                        self.fail('Missing property', obj, pointer)
+                    for name in set(dependencies) - set(obj.keys()):
+                        self.fail('Missing property', obj, pointer_join(pointer, name))  # noqa
                 else:
                     dependencies(obj)
         return obj
@@ -436,7 +433,9 @@ class Draft04Validator(Validator):
             elif isinstance(items, (list, tuple)):
                 additionals = self.attrs['additional_items']
                 validators = items
-                for index, element in enumerate(obj):
+
+                validated = list(obj)
+                for index, element in enumerate(validated):
                     with self.catch_fail():
                         try:
                             validator = validators[index]
@@ -446,10 +445,12 @@ class Draft04Validator(Validator):
                             elif additionals is False:
                                 self.fail('Forbidden value',
                                           obj,
-                                          pointer=pointer_join(self.uri, index))
+                                          pointer=pointer_join(self.uri, index))  # noqa
                                 continue
                             validator = additionals
-                        obj[index] = validator(element, pointer_join(pointer, index))  # noqa
+                        validated[index] = \
+                            validator(element, pointer_join(pointer, index))  # noqa
+                obj = obj.__class__(validated)
                 return obj
             else:
                 raise NotImplementedError(items)
@@ -549,7 +550,7 @@ class Draft04Validator(Validator):
                     pass
             if not validated:
                 self.fail('Validates noone', obj)
-            if validated == 1:
+            elif validated == 1:
                 return validated_obj
             else:
                 self.fail('Validates more than once', obj)
@@ -594,7 +595,8 @@ class Draft04Validator(Validator):
             return obj
 
         if additionals is False:
-            self.fail('Forbidden additional properties', obj, pointer)
+            for name in pending:
+                self.fail('Forbidden property', obj, pointer_join(pointer, name))  # noqa
             return obj
 
         validator = additionals
@@ -607,13 +609,13 @@ class Draft04Validator(Validator):
         if 'required' in self.attrs:
             for name in self.attrs['required']:
                 if name not in obj:
-                    self.fail('Missing property', obj, pointer)
+                    self.fail('Missing property', obj, pointer_join(pointer, name))  # noqa
         return obj
 
     def validate_type(self, obj, pointer=None):
         if 'type' in self.attrs:
             types = self.attrs['type']
-            if isinstance(types, string_types):
+            if isinstance(types, str):
                 types = [types]
 
             for t in types:
