@@ -6,20 +6,19 @@
 
 from __future__ import absolute_import
 
-__all__ = []
-
+import ipaddress
 import logging
 import re
 import time
 from copy import deepcopy
 from decimal import Decimal
 from datetime import tzinfo, timedelta, datetime, date
-from six import text_type
-from six import integer_types
-from six.moves.urllib.parse import urlparse
+from urllib.parse import urlparse
 from .exceptions import ValidationError
 
-number_types = (integer_types, float, Decimal)
+__all__ = []
+
+number_types = (int, float, Decimal)
 
 logger = logging.getLogger(__name__)
 
@@ -80,7 +79,6 @@ def uncamel(name):
 class offset(tzinfo):
     def __init__(self, value):
         self.value = value
-        # super(tzinfo, self).__init__()
 
     def utcoffset(self, dt):
         hours, minutes = self.value.split(':', 1)
@@ -151,7 +149,7 @@ def validate_utc_datetime(obj):
 def validate_utc_date(obj):
     try:
         time.strptime(obj, '%Y-%m-%d')
-    except ValueError:
+    except (TypeError, ValueError):
         raise ValidationError('{!r} is not a valid date', obj)
     return obj
 
@@ -159,7 +157,7 @@ def validate_utc_date(obj):
 def validate_utc_time(obj):
     try:
         time.strptime(obj, '%H:%M:%S')
-    except ValueError:
+    except (TypeError, ValueError):
         raise ValidationError('{!r} is not a valid time', obj)
     return obj
 
@@ -168,7 +166,7 @@ def validate_utc_millisec(obj):
     try:
         if not isinstance(obj, number_types):
             raise TypeError
-        datetime.utcfromtimestamp(obj/1000)
+        datetime.utcfromtimestamp(obj / 1000)
     except (TypeError, ValueError):
         raise ValidationError('{!r} is not a valid utc millis', obj)
     return obj
@@ -199,11 +197,8 @@ def validate_hostname(obj):
 
 def validate_ipv4(obj):
     try:
-        import ipaddress
-        obj = text_type(obj)
+        obj = str(obj)
         ipaddress.IPv4Address(obj)
-    except ImportError:
-        raise ValidationError('IPv4 relies on ipaddress package', obj)
     except (ipaddress.AddressValueError, ipaddress.NetmaskValueError):
         raise ValidationError('{!r} does not appear to '
                               'be an IPv4 address'.format(obj))
@@ -212,11 +207,8 @@ def validate_ipv4(obj):
 
 def validate_ipv6(obj):
     try:
-        import ipaddress
-        obj = text_type(obj)
+        obj = str(obj)
         ipaddress.IPv6Address(obj)
-    except ImportError:
-        raise ValidationError('IPv6 relies on ipaddress package', obj)
     except (ipaddress.AddressValueError, ipaddress.NetmaskValueError):
         raise ValidationError('{!r} does not appear to '
                               'be an IPv6 address'.format(obj))
